package com.sy.trustypay;

import com.sy.trustypay.util.TrustyClient;
import com.sy.trustypay.util.TrustyPayConfig;
import com.sy.trustypay.util.TrustyPayConstants;
import com.sy.trustypay.util.TrustyPayConstants.SignType;
import com.sy.trustypay.util.TrustyPayUtil;

import java.util.Map;

import static com.sy.trustypay.util.TrustyPayConstants.*;

/**
 * @author longshiwei
 * @date 2021/12/13 14:53
 **/
public class TrustyPay {


    private TrustyPayConfig config;

    private TrustyClient trustyClient;

    public TrustyPay(TrustyPayConfig config) {
        this.config = config;
        trustyClient = new TrustyClient(config);
    }


    /**
     * check trustypay config
     *
     * @return void
     * @date 2021/12/13 15:05
     */
    private void checkPayConfig() throws Exception {
        if (this.config == null) {
            throw new Exception("config is null");
        }
        if (this.config.getAppID() == null || this.config.getAppID().trim().length() == 0) {
            throw new Exception("appid in config is empty");
        }
        if (this.config.getMchID() == null || this.config.getMchID().trim().length() == 0) {
            throw new Exception("appid in config is empty");
        }
        if (this.config.getHttpConnectTimeoutMs() < 10) {
            throw new Exception("http connect timeout is too small");
        }
        if (this.config.getHttpReadTimeoutMs() < 10) {
            throw new Exception("http read timeout is too small");
        }

    }

    /**
     * 向 Map 中添加 appid、mch_id、nonce_str、sign_type、sign <br>
     * 该函数适用于商户适用于统一下单等接口，不适用于红包、代金券接口
     *
     * @param reqData
     * @return
     * @throws Exception
     */
    public Map<String, String> fillRequestData(Map<String, String> reqData) throws Exception {
        reqData.put(FIELD_APPID, config.getAppID());
        reqData.put(FIELD_MCHNT_ID, config.getMchID());
        reqData.put(FIELD_NONCE_STR, TrustyPayUtil.generateNonceStr());
        if (SignType.MD5.equals(config.getSignType())) {
            reqData.put(FIELD_SIGN_TYPE, TrustyPayConstants.MD5);
        } else if (SignType.HMACSHA256.equals(config.getSignType())) {
            reqData.put(FIELD_SIGN_TYPE, TrustyPayConstants.HMACSHA256);
        }
        reqData.put(FIELD_SIGN, TrustyPayUtil.generateSignature(reqData, config.getKey(), config.getSignType()));
        return reqData;
    }

    /**
     * 作用：提交支付申请下单<br>
     * 场景：申请下单
     *
     * @param reqData 向trustypay post的请求数据
     * @return API返回数据
     * @throws Exception
     */
    public Map<String, String> unifiedorder(Map<String, String> reqData) throws Exception {
        String url = TrustyPayConstants.UNIFIEDORDER_URL_SUFFIX;
        return this.post(url, reqData, this.config.getHttpConnectTimeoutMs(), this.config.getHttpReadTimeoutMs());
    }


    /**
     * 作用：查询支付结果<br>
     * 场景：查询支付结果
     *
     * @param reqData 向trustypay post的请求数据
     * @return API返回数据
     * @throws Exception
     */
    public Map<String, String> query(Map<String, String> reqData) throws Exception {
        String url = TrustyPayConstants.ORDERQUERY_URL_SUFFIX;
        return this.post(url, reqData, this.config.getHttpConnectTimeoutMs(), this.config.getHttpReadTimeoutMs());
    }

    /**
     * 作用：查询支付渠道<br>
     * 场景：查询支付渠道
     *
     * @param reqData 向trustypay post的请求数据
     * @return API返回数据
     * @throws Exception
     */
    public Map<String, String> payChannels(Map<String, String> reqData) throws Exception {
        String url = TrustyPayConstants.PAY_CHANNELS;
        return this.post(url, reqData, this.config.getHttpConnectTimeoutMs(), this.config.getHttpReadTimeoutMs());
    }

    /**
     * 作用：查询支付结果<br>
     * 场景：查询支付结果
     *
     * @param reqData 向trustypay post的请求数据
     * @return API返回数据
     * @throws Exception
     */
    public Map<String, String> refund(Map<String, String> reqData) throws Exception {
        String url = TrustyPayConstants.REFUND_URL_SUFFIX;
        return this.post(url, reqData, this.config.getHttpConnectTimeoutMs(), this.config.getHttpReadTimeoutMs());
    }

    /**
     * 作用：查询账单结果<br>
     * 场景：查询账单结果
     *
     * @param reqData 向trustypay post的请求数据
     * @return API返回数据
     * @throws Exception
     */
    public Map<String, String> queryBills(Map<String, String> reqData) throws Exception {
        String url = TrustyPayConstants.BILLS_URL_SUFFIX;
        return this.post(url, reqData, this.config.getHttpConnectTimeoutMs(), this.config.getHttpReadTimeoutMs());
    }

    /**
     * 作用：查询结算结果<br>
     * 场景：查询结算结果
     *
     * @param reqData 向trustypay post的请求数据
     * @return API返回数据
     * @throws Exception
     */
    public Map<String, String> querySettlement(Map<String, String> reqData) throws Exception {
        String url = TrustyPayConstants.SETTLEMENT_URL_SUFFIX;
        return this.post(url, reqData, this.config.getHttpConnectTimeoutMs(), this.config.getHttpReadTimeoutMs());
    }

    /**
     * 作用：提交刷卡支付<br>
     * 场景：刷卡支付
     *
     * @param reqData          向trustypay post的请求数据
     * @param connectTimeoutMs 连接超时时间，单位是毫秒
     * @param readTimeoutMs    读超时时间，单位是毫秒
     * @return API返回数据
     * @throws Exception
     */
    private Map<String, String> post(String url, Map<String, String> reqData, int connectTimeoutMs, int readTimeoutMs) throws Exception {

        //fillRequestData 向 Map 中添加 appid、mch_id、nonce_str、sign_type、sign <br>
        Map<String, String> respMap = this.trustyClient.request(url, this.fillRequestData(reqData), connectTimeoutMs, readTimeoutMs);
        return this.processResponse(respMap);
    }


    /**
     * 处理 HTTPS API返回数据，转换成Map对象。return_code为SUCCESS时，验证签名。
     *
     * @param respData API返回的JSON格式数据
     * @return Map类型数据
     * @throws Exception
     */
    private Map<String, String> processResponse(Map<String, String> respData) throws Exception {

        String return_code;
        if (respData.containsKey(FIELD_RETURN_CODE)) {
            return_code = respData.get(FIELD_RETURN_CODE);
        } else {
            throw new Exception(String.format("No `return_code` in json: %s", respData));
        }

        if (return_code.equals(TrustyPayConstants.SUCCESS)) {
            // 返回数据的签名方式和请求中给定的签名方式是一致的
            if (TrustyPayUtil.isSignatureValid(respData, this.config.getKey(), this.config.getSignType())) {
                return respData;
            } else {
                throw new Exception(String.format("Invalid sign value in json: %s", respData));
            }
        } else {
            return respData;
        }
    }

}
