package com.sy.trustypay.util;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.DefaultHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.BasicHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import java.io.InputStream;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.util.Map;

/**
 * @author longshiwei
 * @date 2021/12/13 15:17
 **/
public class TrustyClient {

    private TrustyPayConfig config;

    public TrustyClient(TrustyPayConfig config) {
        this.config = config;
    }

    public Map<String, String> request(String urlSuffix, Map<String, String> reqData, int connectTimeoutMs, int readTimeoutMs) throws Exception {
        long elapsedTimeMillis = 0;
        Exception exception = null;
        long startTimestampMs = TrustyPayUtil.getCurrentTimestampMs();
        boolean firstHasErr = false;

        String url = config.getDomain() + urlSuffix;
        try {
            BasicHttpClientConnectionManager connManager;
            if (config.getUseCert()) {
                // 证书
                char[] password = config.getMchID().toCharArray();
                InputStream certStream = config.getCertStream();
                KeyStore ks = KeyStore.getInstance("PKCS12");
                ks.load(certStream, password);

                // 实例化密钥库 & 初始化密钥工厂
                KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
                kmf.init(ks, password);

                // 创建 SSLContext
                SSLContext sslContext = SSLContext.getInstance("TLS");
                sslContext.init(kmf.getKeyManagers(), null, new SecureRandom());

                SSLConnectionSocketFactory sslConnectionSocketFactory = new SSLConnectionSocketFactory(
                        sslContext,
                        new String[]{"TLSv1"},
                        null,
                        new DefaultHostnameVerifier());

                connManager = new BasicHttpClientConnectionManager(
                        RegistryBuilder.<ConnectionSocketFactory>create()
                                .register("http", PlainConnectionSocketFactory.getSocketFactory())
                                .register("https", sslConnectionSocketFactory)
                                .build(),
                        null,
                        null,
                        null
                );
            } else {
                connManager = new BasicHttpClientConnectionManager(
                        RegistryBuilder.<ConnectionSocketFactory>create()
                                .register("http", PlainConnectionSocketFactory.getSocketFactory())
                                .register("https", SSLConnectionSocketFactory.getSocketFactory())
                                .build(),
                        null,
                        null,
                        null
                );
            }
            HttpClient httpClient = HttpClientBuilder.create()
                    .setConnectionManager(connManager)
                    .build();


            HttpPost httpPost = new HttpPost(url);

            RequestConfig requestConfig = RequestConfig.custom().setSocketTimeout(readTimeoutMs).setConnectTimeout(connectTimeoutMs).build();
            httpPost.setConfig(requestConfig);

            StringEntity postEntity = new StringEntity(JSON.toJSONString(reqData), "UTF-8");
            httpPost.addHeader("Content-Type", "application/json");
            httpPost.addHeader("User-Agent", TrustyPayConstants.USER_AGENT + " " + config.getMchID());
            httpPost.setEntity(postEntity);

            HttpResponse httpResponse = httpClient.execute(httpPost);
            HttpEntity httpEntity = httpResponse.getEntity();
            String respData = EntityUtils.toString(httpEntity, "UTF-8");
            elapsedTimeMillis = TrustyPayUtil.getCurrentTimestampMs() - startTimestampMs;

            Map<String, String> params = JSONObject.parseObject(respData, new TypeReference<Map<String, String>>() {
            });

            return params;
        } catch (UnknownHostException ex) {  // dns 解析错误，或域名不存在
            exception = ex;
            firstHasErr = true;
            elapsedTimeMillis = TrustyPayUtil.getCurrentTimestampMs() - startTimestampMs;
            TrustyPayUtil.getLogger().warn("UnknownHostException for domainInfo {}" + url);
        } catch (ConnectTimeoutException ex) {
            exception = ex;
            firstHasErr = true;
            elapsedTimeMillis = TrustyPayUtil.getCurrentTimestampMs() - startTimestampMs;
            TrustyPayUtil.getLogger().warn("connect timeout happened for domainInfo {}" + url);
        } catch (SocketTimeoutException ex) {
            exception = ex;
            firstHasErr = true;
            elapsedTimeMillis = TrustyPayUtil.getCurrentTimestampMs() - startTimestampMs;
            TrustyPayUtil.getLogger().warn("timeout happened for domainInfo " + url);
        } catch (Exception ex) {
            exception = ex;
            elapsedTimeMillis = TrustyPayUtil.getCurrentTimestampMs() - startTimestampMs;
        } finally {
            // 上报异常 TODO
        }

        TrustyPayUtil.getLogger().warn("request trusty pay resutl :  HasErr=" + firstHasErr + ",elapsedTimeMillis=" + elapsedTimeMillis);
        throw exception;
    }
}
